import setuptools

VERSION = "2.1.6a"

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cndi",
    version=VERSION,
    author="Mayank Shinde",
    author_email="mayank31313@gmail.com",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)